package ch.psi.daq.queryrest.response.smile;

import java.io.OutputStream;
import java.util.List;
import java.util.Map.Entry;
import java.util.stream.Stream;

import javax.servlet.ServletResponse;

import org.apache.commons.lang3.tuple.Triple;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.smile.SmileFactory;

import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.backend.BackendQuery;
import ch.psi.daq.domain.query.response.Response;
import ch.psi.daq.queryrest.config.QueryRestConfig;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;
import ch.psi.daq.queryrest.response.json.JSONResponseStreamWriter;

/**
 * Takes a Java 8 stream and writes it to the output stream provided by the {@link ServletResponse}
 * of the current request.
 */
public class SmileResponseStreamWriter implements ResponseStreamWriter, ApplicationContextAware {

   private ObjectMapper mapper;
   private SmileFactory factory;

   @Override
   public void setApplicationContext(ApplicationContext context) throws BeansException {
      final Backend backend = context.getBean(DomainConfig.BEAN_NAME_BACKEND_DEFAULT, Backend.class);
      context = backend.getApplicationContext();

      mapper = context.getBean(DomainConfig.BEAN_NAME_OBJECT_MAPPER, ObjectMapper.class);
      factory = context.getBean(QueryRestConfig.BEAN_NAME_SMILE_FACTORY, SmileFactory.class);
   }

   @Override
   public void respond(final List<Entry<DAQQueryElement, Stream<Triple<BackendQuery, ChannelName, ?>>>> results,
         final OutputStream out, final Response response) throws Exception {
      JSONResponseStreamWriter.respond(factory, mapper, results, out, response);
   }
}
